package com.example.user.myapp;

import android.app.IntentService;
import android.content.Intent;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;

import com.ghgande.j2mod.modbus.ModbusException;
import com.ghgande.j2mod.modbus.facade.ModbusTCPMaster;
import com.ghgande.j2mod.modbus.io.ModbusTransaction;
import com.ghgande.j2mod.modbus.msg.ModbusResponse;
import com.ghgande.j2mod.modbus.net.TCPMasterConnection;
import com.ghgande.j2mod.modbus.util.BitVector;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

/**
 * An {@link IntentService} subclass for handling asynchronous task requests in
 * a service on a separate handler thread.
 * <p>
 * TODO: Customize class - update intent actions, extra parameters and static
 * helper methods.
 */
public class MyIntentService extends IntentService {
    // TODO: Rename actions, choose action names that describe tasks that this
    // IntentService can perform, e.g. ACTION_FETCH_NEW_ITEMS
  //  private static final String ACTION_FOO = "com.example.user.myapp.action.FOO";
    private static final String ACTION_READ_COIL = "read.coil";

    private ModbusTCPMaster master;

    // TODO: Rename parameters
    private static final String EXTRA_IP_ADDRESS = "extra.ip.address";
    private static final String EXTRA_PORT = "extra.ip.port";
    private static final String EXTRA_REF = "extra.ref";
    private static final String EXTRA_COUNT = "extra.count";
    private static final String EXTRA_INTENT_NAME = "extra.intent.name";

 //   private static final String EXTRA_PARAM2 = "com.example.user.myapp.extra.PARAM2";

    public MyIntentService() {
        super("MyIntentService");
    }

    private void sendBooleanListToActivity(List<Boolean> booleanList, String intentName) {
        Intent intent = new Intent(intentName);
        Bundle bundle = new Bundle();

        bundle.putBooleanArray("values", toPrimitiveArray(booleanList));
        intent.putExtras(bundle);
        sendLocationBroadcast(intent);
    }

    private void sendLocationBroadcast(Intent intent) {
        LocalBroadcastManager.getInstance(this).sendBroadcast(intent);
    }

    public void readCoilAction (Context context, String ipAddress, int port, int ref, int count, String intentReceiveName) {
        try {
            master = new ModbusTCPMaster(ipAddress, port);
            master.connect();

            BitVector bv = master.readCoils(ref,count);
            List<Boolean> listBooleanBit = new ArrayList<>();
            for (int i = 0; i < count; i++) {
                listBooleanBit.add(bv.getBit(i));
            }
            sendBooleanListToActivity(listBooleanBit, intentReceiveName);
            master.disconnect();
        } catch (Exception e) {
            System.out.println("Exception in reading coil " + e);
        }
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        GlobalState state = (GlobalState) getApplicationContext();
        if (intent != null) {
            final String action = intent.getAction();
            final String ipAddress = intent.getStringExtra(EXTRA_IP_ADDRESS);
            final int port = intent.getIntExtra(EXTRA_PORT, 502);
            final int ref = intent.getIntExtra(EXTRA_REF, 8192);
            final int count = intent.getIntExtra(EXTRA_COUNT, 8);
            final String intentName = intent.getStringExtra(EXTRA_INTENT_NAME);

            switch(action) {
                //case ACTION_FOO :
                //    handleActionFoo(param1, param2);
                //   break;
                case ACTION_READ_COIL :
                    handleReadCoilAction(ipAddress, port, ref, count, intentName);
            }
        }
    }

    /**
     * Handle action Baz in the provided background thread with the provided
     * parameters.
     */
    private void handleReadCoilAction(String ipAddress, int port, int ref, int count, String intentName) {
        readCoilAction(this, ipAddress, port, ref, count, intentName);
    }

    private boolean[] toPrimitiveArray(final List<Boolean> booleanList) {
        final boolean[] primitives = new boolean[booleanList.size()];
        int index = 0;
        for (Boolean object : booleanList) {
            primitives[index++] = object;
        }
        return primitives;
    }
}
